@@ -15,19 +15,17 @@ defaults.
 # New series numbers assigned using this tweak are always integer values, except
 # if a constant non-integer is specified.
 # Possible values are:
-# next - First available integer larger than the largest existing number
-# first_free - First available integer larger than 0
-# next_free - First available integer larger than the smallest existing number
-# last_free - First available integer smaller than the largest existing number
-#             Return largest existing + 1 if no free number is found
-# const - Assign the number 1 always
-# no_change - Do not change the series index
-# a number - Assign that number always. The number is not in quotes. Note that
-#            0.0 can be used here.
+#   next - First available integer larger than the largest existing number
+#   first_free - First available integer larger than 0
+#   next_free - First available integer larger than the smallest existing number
+#   last_free - First available integer smaller than the largest existing number. Return largest existing + 1 if no free number is found
+#   const - Assign the number 1 always
+#   no_change - Do not change the series index
+#   a number - Assign that number always. The number is not in quotes. Note that 0.0 can be used here.
 # Examples:
-# series_index_auto_increment = 'next'
-# series_index_auto_increment = 'next_free'
-# series_index_auto_increment = 16.5
+#   series_index_auto_increment = 'next'
+#   series_index_auto_increment = 'next_free'
+#   series_index_auto_increment = 16.5
 #
 # Set the use_series_auto_increment_tweak_when_importing tweak to True to
 # use the above values when importing/adding books. If this tweak is set to
@@ -51,7 +49,7 @@ use_series_auto_increment_tweak_when_importing = False
 authors_completer_append_separator = False
 
 #: Author sort name algorithm
-# The algorithm used to copy author to author_sort
+# The algorithm used to copy author to author_sort.
 # Possible values are:
 #  invert: use "fn ln" -> "ln, fn"
 #  copy  : copy author to author_sort without modification
@@ -132,11 +130,11 @@ tag_browser_category_order = {'*':1}
 
 
 #: Specify columns to sort the booklist by on startup
-# Provide a set of columns to be sorted on when calibre starts
-#  The argument is None if saved sort history is to be used
-#  otherwise it is a list of column,order pairs. Column is the
-#  lookup/search name, found using the tooltip for the column
-#  Order is 0 for ascending, 1 for descending
+# Provide a set of columns to be sorted on when calibre starts.
+# The argument is None if saved sort history is to be used
+# otherwise it is a list of column,order pairs. Column is the
+# lookup/search name, found using the tooltip for the column
+# Order is 0 for ascending, 1 for descending.
 # For example, set it to [('authors',0),('title',0)] to sort by
 # title within authors.
 sort_columns_at_startup = None
@@ -261,7 +259,7 @@ per_language_title_sort_articles = {
         'ell'  : (r'O\s+', r'I\s+', r'To\s+', r'Ta\s+', r'Tus\s+', r'Tis\s+',
                   r"'Enas\s+", r"'Mia\s+", r"'Ena\s+", r"'Enan\s+", ),
         # Hungarian
-        'hun'  : (r'A\s+', 'Az\s+', 'Egy\s+',),
+        'hun'  : (r'A\s+', r'Az\s+', r'Egy\s+',),
 }
 default_language_for_title_sort = None
 title_sort_articles=r'^(A|The|An)\s+'
@@ -310,20 +308,20 @@ auto_connect_to_folder = ''
 # '{value} {category:|(|)}'
 # Examples: The first three examples assume that the second tweak
 # has not been changed.
-# 1: I want three series columns to be merged into one set of collections. The
-# column lookup names are 'series', '#series_1' and '#series_2'. I want nothing
-# in the parenthesis. The value to use in the tweak value would be:
+#  1: I want three series columns to be merged into one set of collections. The
+#  column lookup names are 'series', '#series_1' and '#series_2'. I want nothing
+#  in the parenthesis. The value to use in the tweak value would be:
 #    sony_collection_renaming_rules={'series':'', '#series_1':'', '#series_2':''}
-# 2: I want the word '(Series)' to appear on collections made from series, and
-# the word '(Tag)' to appear on collections made from tags. Use:
+#  2: I want the word '(Series)' to appear on collections made from series, and
+#  the word '(Tag)' to appear on collections made from tags. Use:
 #    sony_collection_renaming_rules={'series':'Series', 'tags':'Tag'}
-# 3: I want 'series' and '#myseries' to be merged, and for the collection name
-# to have '(Series)' appended. The renaming rule is:
+#  3: I want 'series' and '#myseries' to be merged, and for the collection name
+#  to have '(Series)' appended. The renaming rule is:
 #    sony_collection_renaming_rules={'series':'Series', '#myseries':'Series'}
-# 4: Same as example 2, but instead of having the category name in parentheses
-# and appended to the value, I want it prepended and separated by a colon, such
-# as in Series: Darkover. I must change the template used to format the category name
-# The resulting two tweaks are:
+#  4: Same as example 2, but instead of having the category name in parentheses
+#  and appended to the value, I want it prepended and separated by a colon, such
+#  as in Series: Darkover. I must change the template used to format the category name
+#  The resulting two tweaks are:
 #    sony_collection_renaming_rules={'series':'Series', 'tags':'Tag'}
 #    sony_collection_name_template='{category:||: }{value}'
 sony_collection_renaming_rules={}
@@ -362,10 +360,13 @@ add_new_book_tags_when_importing_books = False
 # Defaults:
 #    content_server_will_display = ['*']
 #    content_server_wont_display = []
+#
 # Examples:
+#
 # To display only the custom fields #mytags and #genre:
 #   content_server_will_display = ['#mytags', '#genre']
 #   content_server_wont_display = []
+#
 # To display all fields except #mycomments:
 #   content_server_will_display = ['*']
 #   content_server_wont_display['#mycomments']
@@ -393,7 +394,7 @@ sort_dates_using_visible_fields = False
 cover_trim_fuzz_value = 10
 
 #: Control behavior of the book list
-# You can control the behavior of double clicks on the books list.
+# You can control the behavior of double clicks and pressing enter on the books list.
 # Choices: open_viewer, do_nothing,
 # edit_cell, edit_metadata. Selecting anything other than open_viewer has the
 # side effect of disabling editing a field using a single click.
@@ -402,6 +403,7 @@ cover_trim_fuzz_value = 10
 # You can also control whether the book list scrolls horizontal per column or
 # per pixel. Default is per column.
 doubleclick_on_library_view = 'open_viewer'
+enter_key_behavior = 'do_nothing'
 horizontal_scrolling_per_column = True
 
 #: Language to use when sorting
@@ -425,7 +427,7 @@ metadata_single_use_2_cols_for_custom_fields = True
 #: Order of custom column(s) in edit metadata
 # Controls the order that custom columns are listed in edit metadata single
 # and bulk. The columns listed in the tweak are displayed first and in the
-# order provided. Any columns not listed are dislayed after the listed ones,
+# order provided. Any columns not listed are displayed after the listed ones,
 # in alphabetical order. Do note that this tweak does not change the size of
 # the edit widgets. Putting comments widgets in this list may result in some
 # odd widget spacing when using two-column mode.
@@ -555,7 +557,14 @@ content_server_thumbnail_compression_quality = 75
 cover_drop_exclude = ()
 
 #: Show the Saved searches box in the Search bar
-# In newer version of calibre, only a button that allows you to add a new Saved
-# search is shown in the Search bar. If you would like to have the old
-# Saved searches box with its two buttons back, set this tweak to True.
+# In newer versions of calibre, only a single button that allows you to add a
+# new Saved search is shown in the Search bar. If you would like to have the
+# old Saved searches box with its two buttons back, set this tweak to True.
 show_saved_search_box = False
+
+#: Exclude fields when copy/pasting metadata
+# You can ask calibre to not paste some metadata fields when using the
+# Edit metadata->Copy metadata/Paste metadata actions. For example,
+# exclude_fields_on_paste = ['cover', 'timestamp', '#mycolumn']
+# to prevent pasting of the cover, Date and custom column, mycolumn.
+exclude_fields_on_paste = []
